// 67d7842dbbe25473c3c32b93c0da8047785f30d78e8a024de1b57352245f9689
/*******************************************************************************
 * Copyright (c) 2007, 2014 Wind River Systems, Inc. and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * Contributors:
 *     Wind River Systems - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * Modified by Xilinx,Inc
 * Copyright 2022 Xilinx, Inc. All Rights Reserved. Modifications
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 *******************************************************************************/

package org.eclipse.tcf.internal.dap;

import org.apache.commons.lang3.tuple.ImmutablePair;
import org.eclipse.tcf.services.ILineNumbers;

import java.io.IOException;
import java.nio.file.Paths;
import java.util.List;

/**
 * TCF source lookup director.
 * For TCF source lookup there is one source lookup participant.
 */
public class TCFSourceLookupDirector {

//    private static Set<String> fSupportedContainerTypes;
//    static {
//        fSupportedContainerTypes = new HashSet<String>();
//        // Bug 440538:
//        // Only CDT path mapping source containers are currently supported
//        fSupportedContainerTypes.add("org.eclipse.cdt.debug.core.containerType.mapping");
//    };

    public static String lookup(final TCFLaunch launch, Object element) {
        if (element instanceof ILineNumbers.CodeArea) {
            String compiledPath = TCFSourceLookupParticipant.toFileName((ILineNumbers.CodeArea)element);
            List<ImmutablePair<String, String>> pathMapEntries = launch.getDebugAdapterConfiguration().getPathMapEntriesList();
            String  source_element = null;
            source_element =  getSourceElement(compiledPath,pathMapEntries);
            return source_element;
        }
        return "";


//        ISourceLocator locator = launch.getSourceLocator();
//        if (locator instanceof ISourceLookupDirector) {
//            source_element = ((ISourceLookupDirector)locator).getSourceElement(element);
//        }
//        else if (element instanceof IStackFrame) {
//            source_element = locator.getSourceElement((IStackFrame)element);
//        }
//        if (source_element == null && element instanceof String) {
//            /* Try to lookup the element using target side path mapping rules */
//            final String str = (String)element;
//            String key = str;
//            if (ctx != null) key = ctx + "::" + str;
//            Map<String,IStorage> map = launch.getTargetPathMappingCache();
//            synchronized (map) {
//                if (map.containsKey(str)) return map.get(key);
//            }
//            IStorage storage = new TCFTask<IStorage>(launch.getChannel()) {
//                public void run() {
//                    TCFDataCache<IPathMap.PathMapRule[]> cache = launch.getTargetPathMap();
//                    if (cache != null) {
//                        if (!cache.validate(this)) return;
//                        IPathMap.PathMapRule[] data = cache.getData();
//                        if (data != null) {
//                            for (IPathMap.PathMapRule r : data) {
//                                final String query = r.getContextQuery();
//                                if (query != null && query.length() > 0 && !query.equals("*")) {
//                                    if (ctx == null) continue;
//                                    TCFDataCache<String[]> q_cache = launch.getContextQuery(query);
//                                    if (q_cache == null) continue;
//                                    if (!q_cache.validate(this)) return;
//                                    String[] q_data = q_cache.getData();
//                                    if (q_data == null) continue;
//                                    boolean ok = false;
//                                    for (String id : q_data) {
//                                        if (ctx.equals(id)) ok = true;
//                                    }
//                                    if (!ok) continue;
//                                }
//                                String fnm = TCFSourceLookupParticipant.toFileName(r, str);
//                                if (fnm == null) continue;
//                                File file = new File(fnm);
//                                if (file.isAbsolute() && file.exists() && file.isFile()) {
//                                    done(new LocalFileStorage(file));
//                                    return;
//                                }
//                            }
//                        }
//                    }
//                    done(null);
//                }
//            }.getE();
//            if (storage != null) {
//                /* Map to workspace resource */
//                IPath path = storage.getFullPath();
//                if (path != null) {
//                    URI uri = URIUtil.toURI(path);
//                    IFile[] arr = ResourcesPlugin.getWorkspace().getRoot().findFilesForLocationURI(uri);
//                    if (arr != null && arr.length > 0) {
//                        for (IFile resource : arr) {
//                            if (resource.isAccessible()) {
//                                storage = resource;
//                                break;
//                            }
//                        }
//                    }
//                }
//            }
//            synchronized (map) {
//                map.put(key, storage);
//            }
//            source_element = storage;
//        }
//        return source_element;
    }

    private static String getSourceElement(String  compiledPath, List<ImmutablePair<String, String>> pathMapEntries) {
        String realCompiledPath = null;
        if( Paths.get(compiledPath).toFile().exists() ) {
            try {
                realCompiledPath = Paths.get(compiledPath).toRealPath().toString();
            } catch (IOException e) {
                e.printStackTrace();
                realCompiledPath = Paths.get(compiledPath).toAbsolutePath().toString();
            }
        } else {
            realCompiledPath = Paths.get(compiledPath).toAbsolutePath().toString();
        }
        for( ImmutablePair<String,String> pathMapEntry : pathMapEntries ){
            if( realCompiledPath.startsWith(pathMapEntry.left)){
                return Paths.get(pathMapEntry.right,realCompiledPath.substring(pathMapEntry.left.length())).toAbsolutePath().toString();
            }
        }
        return realCompiledPath;
    }

//    public void initializeParticipants() {
//        addParticipants(new ISourceLookupParticipant[] { new TCFSourceLookupParticipant() });
//    }

    /* (non-Javadoc)
     * @see org.eclipse.debug.core.sourcelookup.ISourceLookupDirector#supportsSourceContainerType(org.eclipse.debug.core.sourcelookup.ISourceContainerType)
     */
//    @Override
//    public boolean supportsSourceContainerType(ISourceContainerType type) {
//        return fSupportedContainerTypes.contains(type.getId());
//    }
}
